{% load rules %}
{% load projectroles_common_tags %}

<div class="btn-group sodar-list-btn-group sodar-pr-btn-grp-role" role="group" id="sodar-pr-btn-grp-role-{{ assignment_id }}">
  <button class="btn btn-secondary dropdown-toggle sodar-list-dropdown"
      type="button" data-toggle="dropdown"
      aria-haspopup="true" aria-expanded="false">
    <i class="iconify" data-icon="mdi:gear"></i>
  </button>
  <div class="dropdown-menu dropdown-menu-right">
    <a class="dropdown-item"
       href="{% url 'projectroles:role_update' roleassignment=assignment_id %}">
      <i class="iconify" data-icon="mdi:lead-pencil"></i> Update Member
    </a>
    <a class="dropdown-item text-danger"
       href="{% url 'projectroles:role_delete' roleassignment=assignment_id %}">
      <i class="iconify" data-icon="mdi:account-remove"></i> Remove Member
    </a>
    {% get_history_dropdown obj=user project=project as history_item %}
      {{ history_item|safe }}
  </div>
</div>
